/*  Sunpos

 Takes an FT2 file and adds colums to include Sun RA and dec.

 Robin Corbet, UMBC. 2010-12-09

*/


/* Argument is: 
1) input file name


*/

#include "fitsio.h"  /* required by every program that uses CFITSIO  */
#include <stdio.h>
#include <math.h>
#include <string.h>

main(argc, argv)
int     argc;
char    *argv[];

{
fitsfile *fptr_sc;       
int status, anynul, i, timebin;
int n_points, n_fields;
double *start_time;
float *sun_ra, *sun_dec;
char buf[120], comment[100];
int time_colnum, sun_ra_colnum,sun_dec_colnum;
double jdref = 2451910.5;

double jd, ra, dec;

char spacecraft_file[250];

status = 0;

if(argc != 2){
	fprintf(stdout, "wrong number of arguments\n");
	return(-10);
}

strcpy(spacecraft_file, argv[1]);

fits_open_data(&fptr_sc, spacecraft_file, READWRITE, &status);



/* read spacecraft header to calculate array size */

fits_read_key(fptr_sc, TINT, "NAXIS2",
                  &n_points, comment, &status);

printf("number of points = %d\n", n_points);


fits_read_key(fptr_sc, TINT, "TFIELDS",
                  &n_fields, comment, &status);

printf("number of fields = %d\n", n_fields);




/* Add columns for solar coordinates */

/* First check whether there's already a Sun column */

fits_get_colnum(fptr_sc, CASEINSEN, "RA_SUN",
                    &sun_ra_colnum, &status);

if(status == 0){
	printf("RA_SUN column already exists. File will not be modified.\n");
	fits_close_file(fptr_sc, &status);            /* close the output file */
	return(0);
}
else{
	printf("No RA_SUN column yet. Adding RA_SUN/DEC_SUN columns.\n");
	status = 0;
}

sun_ra_colnum = n_fields + 1;
sun_dec_colnum = sun_ra_colnum + 1;


/* allocate needed array space */

start_time = (double *) calloc (n_points, sizeof(double));


sun_ra = (float *) calloc (n_points, sizeof(float));
sun_dec = (float *) calloc (n_points, sizeof(float));


/* Get the start time values */

fits_get_colnum(fptr_sc, CASEINSEN, "START",
                    &time_colnum, &status);

fits_read_col(fptr_sc, TDOUBLE, time_colnum, 1,
       1, n_points, NULL, start_time, 
	&anynul, &status);


/* calculate Sun RA/dec here */


for (i = 0; i < n_points; i++){

/* convert MET to Julian Day */

	jd = start_time[i]/86400.0 + jdref;

	/* jd = 2444606.5; */

	/* printf("start_time = %g, jd = %g\n", start_time[i], jd); */

	snradc(jd, &ra, &dec);
	sun_ra[i] = ra;
	sun_dec[i] = dec;

	/* printf("ra = %g, dec = %g\n", sun_ra[i], sun_dec[i]); */


}

fits_insert_col(fptr_sc, sun_ra_colnum, "RA_SUN",
                     "E", &status);

fits_insert_col(fptr_sc, sun_dec_colnum, "DEC_SUN",
                     "E", &status);


fits_write_col(fptr_sc, TFLOAT, sun_ra_colnum, 1,
                  1, n_points, sun_ra, &status); 

fits_write_col(fptr_sc, TFLOAT, sun_dec_colnum, 1,
                  1, n_points, sun_dec, &status); 




fits_close_file(fptr_sc, &status);            /* close the output file */


fits_report_error(stderr, status);  /* print out any error messages */
return (status);
}



/* Hacked FORTRAN code! */

int snradc(jd,ra,dec)
/*
C THIS ROUTINE COMPUTES APPROXIMATE POSITION OF THE SUN FOR SPECIFIED
C       JULIAN DATE.
C ALGORITHM TAKEN FROM 1994 NAUTICAL ALMANAC PAGE C24.
C       ACCURATE TO 0.01 DEGREES FROM 1950 TO 2050
C
C ALL VARIABLES ARE REAL*8
C
C INPUT:
C       JD IS JULIAN DATE

C
C OUTPUT:
C       RA IS RIGHT ASCENSION IN DEGREES (J2000?)
C       DEC IS DECLINATION IN DEGREES
C
*/

double jd, *ra, *dec;

{
double rad, l, g, elon, obl, snl;
double n;

	/* printf("jd = %g\n", jd); */

        rad = 57.29577951;

        n = jd - 2451545.0;

/* l is mean longitude of sun, corrected for aberration */
        l = fmod(280.466 + 0.9856474*n, 360.0);
/* g is mean anomaly */
        g = fmod(357.528 + 0.9856003*n, 360.0);
/* elon is ecliptic longitude */
        elon = l + sin(g/rad)*1.915 + sin(2.0 * g/rad)*0.02;
/* e is obliquity of ecliptic */
        obl = 23.440 - (4.0e-7)*n;
        snl = sin(elon/rad);
        *ra = rad * atan2(snl*cos(obl/rad),cos(elon/rad));
        if(*ra < 0.0e0){*ra = *ra + 360.0;}
        *dec = rad * asin(sin(obl/rad)*snl);

	/* printf("in function, ra = %g, dec = %g\n", *ra, *dec); */

        return;
}

